package de.fhw.swtvertiefung.knauf.kuchenzutatnm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.ejb.CreateException;
import javax.ejb.EntityContext;


/**
 * Abstrakte Bean-Klasse fr ein Objekt "KuchenNM". Sie hat genau ein weiteres
 * Feld: "Kuchen-Name". Die ID wird vom Container erzeugt.
 * 
 * Per Container-Managed Relationship wird eine Liste von Zutaten zugefgt.
 * 
 * @author Wolfgang Knauf
 */
public abstract class KuchenNMBean implements javax.ejb.EntityBean
{
  /** Der Entity-Context der Bean */
  private EntityContext entityContext = null;

  /** Der Logger. */
  private static final Logger logger = Logger.getLogger(KuchenNMBean.class.getName());

  /**
   * EntityContext abrufen.
   * 
   * @return Der EntityContext.
   */
  public javax.ejb.EntityContext getEntityContext()
  {
    return entityContext;
  }

  ////////////////////////////////////////////////////////////////////
  //          Methoden aus Interface EntityBean //
  ////////////////////////////////////////////////////////////////////

  /**
   * Bean wird aktiviert.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbActivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    KuchenNMBean.logger.info("ejbActivate");
  }

  /**
   * Bean wird geladen.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbLoad() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    KuchenNMBean.logger.info("ejbLoad");
  }

  /**
   * Bean wird passiviert.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbPassivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    KuchenNMBean.logger.info("ejbPassivate");
  }

  /**
   * Bean wird gelscht.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbRemove() throws javax.ejb.RemoveException, javax.ejb.EJBException, java.rmi.RemoteException
  {
    KuchenNMBean.logger.info("ejbRemove");
  }

  /**
   * Bean wird gespeichert.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbStore() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    KuchenNMBean.logger.info("ejbStore");
  }

  /**
   * Entity-Context wird zurckgesetzt.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void unsetEntityContext() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    this.entityContext = null;
  }

  /**
   * Entity-Context wird gesetzt.
   * 
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void setEntityContext(EntityContext parm1) throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    this.entityContext = parm1;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                   Zugriffsmethoden fr die Felder //
  //////////////////////////////////////////////////////////////////////////////////

  /**
   * Abrufen der Kuchen-ID
   * 
   * @return Die Kuchen-ID
   */
  public abstract Integer getId();

  /**
   * Setzen der Kuchen-Id.
   * 
   * @param int_Id
   *          Die neue ID.
   */
  public abstract void setId(Integer int_Id);

  /**
   * Abrufen des Kuchen-Namens.
   * 
   * @return der Kuchen-Name.
   */
  public abstract String getName();

  /**
   * Setzen des Kuchen-Namens.
   * 
   * @param str_Name
   *          der Name
   */
  public abstract void setName(String str_Name);

  /**
   * Abrufen aller Zutaten dieses Kuchens.
   * 
   * @return Collection von Zutat-Objekten.
   */
  public abstract Collection getZutaten();

  /**
   * Setzen der kompletten Zutaten-Liste des Kuchens. Wird vom Container
   * bernommen.
   * 
   * @param coll_Zutaten
   *          Liste der Zutaten.
   */
  public abstract void setZutaten(Collection coll_Zutaten);

  /**
   * Hinzufgen einer Zutat zu den Zutaten dieses Kuchens.
   * 
   * @param zutat
   *          Die zuzufgende Zutat.
   */
  public void addZutat(ZutatNMLocal zutat)
  {
    Collection collZutaten = this.getZutaten();
    collZutaten.add(zutat);
    KuchenNMBean.logger.info("addZutat: Anzahl Zutaten: " + collZutaten.size());
  }
  
  /**
   * Entfernen einer Zutat zu den Zutaten dieses Kuchens.
   * 
   * @param zutat
   *          Die zuzufgende Zutat.
   */
  public void removeZutat(ZutatNMLocal zutat)
  {
    Collection collZutaten = this.getZutaten();
    collZutaten.remove(zutat);
    KuchenNMBean.logger.info("removeZutat: Anzahl Zutaten: " + collZutaten.size());
  }

  /**
   * Abrufen der Zutatenliste. NUR hierber darf ein Client an die Zutaten
   * kommen. "getZutaten" ist nur fr den Container ?
   * 
   * @return Eine Liste aller Zutaten.
   */
  public ArrayList getZutatenListe()
  {
    ArrayList list = new ArrayList();
    Iterator c = this.getZutaten().iterator();
    while (c.hasNext())
    {
      list.add((ZutatNMLocal) c.next());
    }
    KuchenNMBean.logger.info("getZutatenListe: " + list.size());
    return list;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          SELECT-Methoden //
  //////////////////////////////////////////////////////////////////////////////////

  /**
   * Ermitteln der maximalen Kuchen-ID in Datenbank Der EJB-QL-Befehl wird per
   * Deploytool festgelegt. (select max (....)
   * 
   * @return Maximale ZutatID aus Datenbank.
   */
  public abstract java.lang.Integer ejbSelectMaxId() throws javax.ejb.FinderException;
  /**
   * Von auen verwendbare Methode aus dem Home-Interface: Ermitteln der
   * nchsten verfgbaren Kuchen-ID.
   * 
   * @return Die neue Kuchen-ID, 1 grer als die maximal vorhandene ID. Wenn es
   *         keine gibt, wird "1" zurckgeliefert.
   */
  public Integer ejbHomeGetNextId()
  {
    Integer intMaxID = null;
    try
    {
      intMaxID = this.ejbSelectMaxId();
    }
    catch (javax.ejb.FinderException fex)
    {
      throw new javax.ejb.EJBException("Konnte MaxID fr Kuchen nicht ermitteln", fex);
    }

    if (intMaxID == null)
      intMaxID = new Integer(0);
    //Um 1 erhhen:
    intMaxID = new Integer(intMaxID.intValue() + 1);
    KuchenNMBean.logger.info("ejbHomeGetNextId: " + intMaxID);
    return intMaxID;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          Weitere Methoden //
  //////////////////////////////////////////////////////////////////////////////////

  /**
   * Erzeugen eines Kuchen-Objekts, wobei der Name angegeben wird. Der Key wird
   * automatisch erzeugt.
   * 
   * @param str_Name
   *          Der Name des Kuchens.
   * @return Die ID des Kuchens.
   * @exception CreateException
   *              Fehler beim Erzeugen.
   */
  public Integer ejbCreateByName(String str_Name) throws CreateException
  {
    KuchenNMBean.logger.info("ejbCreateByName: " + str_Name);
    this.setName(str_Name);

    //Key erzeugen:
    this.setId(this.ejbHomeGetNextId());
    return null;
  }

  /**
   * Erzeugen eines Kuchen-Objekts, Name ist noch nicht gegeben. Der Key wird
   * automatisch erzeugt.
   * 
   * @return Die ID des Kuchens.
   * @exception CreateException
   *              Fehler beim Erzeugen.
   */
  public Integer ejbCreate() throws CreateException
  {
    KuchenNMBean.logger.info("ejbCreate");
    //Key erzeugen:
    //this.setId ( ( (KuchenLocalHome) (this.getEntityContext().getEJBLocalHome())).getNextId() );
    this.setId(this.ejbHomeGetNextId());

    return null;
  }

  /**
   * Nach dem Erzeugen eines Kuchen-Objekts, wobei der Name angegeben wird:
   * nichts besonderes zu tun.
   * 
   * @param str_Name
   *          Der Name des Kuchens.
   * @exception CreateException
   *              Fehler beim Erzeugen.
   */
  public void ejbPostCreateByName(String str_Name) throws CreateException
  {
    KuchenNMBean.logger.info("ejbPostCreateByName: " + str_Name);
  }

  /**
   * Nach dem Erzeugen eines Kuchen-Objekts, Name ist noch nicht gegeben.
   * 
   * @exception CreateException
   *              Fehler beim Erzeugen.
   */
  public void ejbPostCreate() throws CreateException
  {
    KuchenNMBean.logger.info("ejbPostCreate");
  }
}
