package de.fhw.swtvertiefung.knauf.kuchenzutatnm;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.ejb.CreateException;
import javax.ejb.EntityContext;

/**
 * Diese Bean bildet eine Zutat ab. 
 * Werte sind: 
 * a) ID 
 * b) Name der Zutat
 * 
 * @author Wolfgang Knauf
 */
public abstract class ZutatNMBean implements javax.ejb.EntityBean
{
  /**Der Entity-Context der Bean */
  private EntityContext entityContext = null;

  /**Der Logger. */
  private static final Logger logger = Logger.getLogger(ZutatNMBean.class.getName());


  /**EntityContext abrufen.
   * @return Der EntityContext.
   */
  public javax.ejb.EntityContext getEntityContext()
  {
    return entityContext;
  }

  ////////////////////////////////////////////////////////////////////
  //          Methoden aus Interface EntityBean                     //
  ////////////////////////////////////////////////////////////////////

  /**Bean wird aktiviert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbActivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("ejbActivate");
  }

  /**Bean wird geladen.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbLoad() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("ejbLoad");
  }

  /**Bean wird passiviert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbPassivate() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("ejbPassivate");
  }

  /**Bean wird gelscht.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbRemove() throws javax.ejb.RemoveException, javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("ejbRemove");
  }

  /**Bean wird gespeichert.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void ejbStore() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("ejbStore");
  }

  /**Entity-Context wird zurckgesetzt.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void unsetEntityContext() throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("unsetEntityContext");
    this.entityContext = null;
  }

  /**Entity-Context wird gesetzt.
   * @throws javax.ejb.EJBException
   * @throws java.rmi.RemoteException
   */
  public void setEntityContext(EntityContext parm1) throws javax.ejb.EJBException, java.rmi.RemoteException
  {
    ZutatNMBean.logger.info("setEntityContext");
    this.entityContext = parm1;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                   Zugriffsmethoden fr die Felder                            //
  //////////////////////////////////////////////////////////////////////////////////

  /**ID der Zutat abrufen.
   * @return Die ID
   */
  public abstract Integer getId();

  /**Setzen der Zutat-ID 
   * @param int_Id Die neue ID
   */
  public abstract void setId(Integer int_Id);

  /**Abrufen des Zutat-Namens.
   * @return Der Zutat-Name
   */
  public abstract String getZutatName();

  /**Setzen des Zutat-Namens.
   * @param str_Name Der neue Name.
   */
  public abstract void setZutatName(String str_Name);

  /**
   * Abrufen aller Kuchen die diese Zutat verwenden.
   * 
   * @return Collection von Kuchen-Objekten.
   */
  public abstract java.util.Collection getKuchen();
  /**
   * Setzen der kompletten Kuchen-Liste die diese Zutat verwenden. Wird vom Container
   * bernommen.
   * 
   * @param collKuchen
   *          Liste der Kuchen.
   */
  public abstract void setKuchen(java.util.Collection collKuchen);
  
  /**
   * Abrufen der Kuchenliste. NUR hierber darf ein Client an die Zutaten
   * kommen. "getKuchen" ist nur fr den Container ?
   * 
   * @return Eine Liste aller Kuchen.
   */
  public ArrayList getKuchenListe()
  {
    ArrayList list = new ArrayList();
    Iterator c = this.getKuchen().iterator();
    while (c.hasNext())
    {
      list.add((KuchenNMLocal) c.next());
    }
    ZutatNMBean.logger.info("getKuchenListe: " + list.size());
    return list;
  }
  
  //////////////////////////////////////////////////////////////////////////////////
  //                          SELECT-Methoden                                     //
  //////////////////////////////////////////////////////////////////////////////////

  /**Ermitteln der maximalen Zutat-ID in Datenbank
   * Der EJB-QL-Befehl wird per Deploytool festgelegt.
   * (select max (....)
   * @return Maximale ZutatID aus Datenbank.
   */
  public abstract Integer ejbSelectMaxId() throws javax.ejb.FinderException;

  /**Von auen verwendbare Methode aus dem Home-Interface: Ermitteln der nchsten verfgbaren
   * Zutat-ID.
   * @return Die neue Zutat-ID, 1 grer als die maximal vorhandene ID. WEnn es keine gibt,
   * wird "1" zurckgeliefert.
   */
  public Integer ejbHomeGetNextId()
  {
    Integer intMaxID = null;
    try
    {
      intMaxID = this.ejbSelectMaxId();
    }
    catch (javax.ejb.FinderException fex)
    {
      throw new javax.ejb.EJBException("Konnte MaxID fr Zutat nicht ermitteln", fex);
    }

    if (intMaxID == null)
      intMaxID = new Integer(0);
    //Um 1 erhhen:
    intMaxID = new Integer(intMaxID.intValue() + 1);
    ZutatNMBean.logger.info("ejbHomeGetNextId: " + intMaxID);

    return intMaxID;
  }

  //////////////////////////////////////////////////////////////////////////////////
  //                          Weitere Methoden                                    //
  //////////////////////////////////////////////////////////////////////////////////

  /**Erzeugen eines Zutat-Objekts, wobei der Name angegeben wird. Der Key wird in der Methode
   * erzeugt.
   * @param str_Name Der Name der Zutat
   * @return Immer NULL.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Integer ejbCreateByName(String str_Name) throws CreateException
  {
    ZutatNMBean.logger.info("ejbCreateByName: " + str_Name);
    this.setZutatName(str_Name);
    this.setId(this.ejbHomeGetNextId());

    return null;
  }

  /**Erzeugen eines Zutat-Objekts, Name und Menge sind noch nicht gegeben. Der Primary Key wird in der Methode
   * erzeugt.
   * @return Die ID
   * @exception CreateException Fehler beim Erzeugen.
   */
  public Integer ejbCreate() throws CreateException
  {
    ZutatNMBean.logger.info("ejbCreate");
    this.setId(this.ejbHomeGetNextId());

    return null;
  }

  /**Nach dem Erzeugen eines Zutat-Objekts, wobei der Name angegeben wird: nichts besonderes zu tun.
   * @param str_Name Der Name der Zutat.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreateByName(String str_Name) throws CreateException
  {
    ZutatNMBean.logger.info("ejbPostCreateByName: " + str_Name);
  }

  /**Nach dem Erzeugen eines Zutat-Objekts, Name und Menge sind noch nicht gegeben.
   * @exception CreateException Fehler beim Erzeugen.
   */
  public void ejbPostCreate() throws CreateException
  {
    ZutatNMBean.logger.info("ejbPostCreate");
  }
}
